#!/usr/bin/perl -w
#
# UnixCryptMD5.pm
# Implementation of Poul-Henning Kemp's crypt_md5 routine in perl.
#
# Copyright (c) 2001 Chris Lightfoot. All rights reserved.
#

=head1 NAME

Crypt::UnixCryptMD5 - Perl 5 module implementing Poul-Henning Kamp's C<crypt_md5>

=cut
package Crypt::UnixCrypt::MD5;

use strict;
use Digest::MD5;

my $rcsid = '$Id:$';

use vars qw(@EXPORT_OK $itoa64 $VERSION);

require Exporter;
*import = \&Exporter::import;
@EXPORT_OK = qw(crypt_md5);

=head1 SYNOPSIS

    use Crypt::UnixCryptMD5 qw(crypt_md5);

    $pwhash = crypt_md5($password, $salt);

=head1 DESCRIPTION

This module implements a pure perl version of Poul-Henning Kamp's C<crypt_md5>
function, which itself is a replacement for the traditional Unix C(crypt)
function. Unlike the traditional password hash, the whole of a password is
significant when calculating a C<crypt_md5> password hash.

Modern Linux and BSD systems allow the use of this style of password in the
system password database, and on such systems perl's standard C<crypt> function
will typically implement C<crypt_md5> as well, selecting which algorithm to
use by examining the first three characters of the password. For C<crypt_md5>
the `magic' string is C<$1$>. On systems which do not implement alternative
password encryption functions, you need this module or something like it.

=head1 FUNCTIONS

=over 4

=cut

$itoa64 = "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
$VERSION = "0.1.0";

sub to64 ($$) {
    my ($num, $len) = @_;
    my $ret = "";
    while (--$len >= 0) {
        $ret .= substr($itoa64, $num & 0x3f, 1);
        $num >>= 6;
    }
    return $ret;
}

=item I<crypt_md5> KEY SALT

Return the C<crypt_md5> password hash of KEY, with the salt SALT. SALT must
consist of between one and eight characters in the set C<[^$]>, followed by
a C<$>. Optionally, it may begin with the `magic' string C<$1$>, which is
ignored.

=cut
sub crypt_md5 ($$) {
    my ($pw, $sp) = @_;
 
    # refine the salt
 
    # skip magic string
    $sp =~ s/^\$1\$//;

    my ($salt) = ($sp =~ /^([^\$]{1,8})\$/);
    return undef unless defined($salt);

    my $ctx = new Digest::MD5;

    # Add password...
    $ctx->add($pw);
    
    # Now magic string
    $ctx->add('$1$');
    
    # Raw salt
    $ctx->add($salt);
    
    my $ctx1 = new Digest::MD5;

    $ctx1->add($pw);
    $ctx1->add($salt);
    $ctx1->add($pw);

    my $final = $ctx1->digest();

    # `Just as many characters of the MD5(pw, salt, pw)'
    for (my $pl = length($pw); $pl > 0; $pl -= 16) {
        $ctx->add(substr($final, 0, $pl > 16 ? 16 : $pl));
    }

    $final = "\0" x 16;

    # Now `something really weird'.
    #
    # I believe that this bit of crypt_md5 has been mis-implemented, but I am
    # not sure.
    for (my $i = length($pw); $i; $i >>= 1) {
        if ($i & 1) {
            $ctx->add("\0");
        } else {
            $ctx->add(substr($pw, 0, 1));
        }
    }

    $final = $ctx->digest();
    

    for (my $i = 0; $i < 1000; ++$i) {
        $ctx1 = undef;
        $ctx1 = new Digest::MD5;
        if ($i & 1) {
            $ctx1->add($pw);
        } else {
            $ctx1->add($final);
        }

        if ($i % 3) {
            $ctx1->add($salt);
        }

        if ($i % 7) {
            $ctx1->add($pw);
        }

        if ($i & 1) {
            $ctx1->add($final);
        } else {
            $ctx1->add($pw);
        }

        $final = $ctx1->digest();
    }

    my @f = unpack("C16", $final);
    $final = "\0" x 16;

    my $ret = '$1$' . $salt . '$';
    
    my $l;

    $l = ($f[ 0]<<16) | ($f[ 6]<<8) | $f[12]; $ret .= to64($l, 4);
    $l = ($f[ 1]<<16) | ($f[ 7]<<8) | $f[13]; $ret .= to64($l, 4);
    $l = ($f[ 2]<<16) | ($f[ 8]<<8) | $f[14]; $ret .= to64($l, 4);
    $l = ($f[ 3]<<16) | ($f[ 9]<<8) | $f[15]; $ret .= to64($l, 4);
    $l = ($f[ 4]<<16) | ($f[10]<<8) | $f[ 5]; $ret .= to64($l, 4);
    $l =                    $f[11]          ; $ret .= to64($l, 2);
}


=back

=head1 BUGS

None known, though the original C<crypt_md5> code contains some peculiarities
which you may wish to review before using this module in anger.

=head1 AUTHOR

Chris Lightfoot <chris@ex-parrot.com>, based upon code by Poul-Henning Kamp,
<phk@login.dknet.dk>.

=head1 COPYRIGHT

The original code is supplied under the following conditions:

=head2 THE BEER-WARE LICENSE (Revision 42):

<phk@login.dknet.dk> wrote [md5_crypt.c].  As long as you retain this notice you
can do whatever you want with this stuff. If we meet some day, and you think
this stuff is worth it, you can buy me a beer in return.   Poul-Henning Kamp

To this I add the following disclaimer of warranty:

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

